package com.sun.corba.se.impl.encoding;

import com.sun.corba.se.impl.orbutil.ORBUtility;
import com.sun.corba.se.pept.transport.ByteBufferPool;
import com.sun.corba.se.pept.transport.Connection;
import java.util.Iterator;
import com.sun.corba.se.pept.encoding.OutputObject;
import com.sun.corba.se.impl.protocol.giopmsgheaders.MessageBase;
import com.sun.corba.se.spi.orb.ORB;

public class BufferManagerWriteCollect extends BufferManagerWrite
{
    private BufferQueue queue;
    private boolean sentFragment;
    private boolean debug;
    
    BufferManagerWriteCollect(ORB orb) {
        super(orb);
        this.queue = new BufferQueue();
        this.sentFragment = false;
        this.debug = false;
        if (orb != null) {
            this.debug = orb.transportDebugFlag;
        }
    }
    
    @Override
    public boolean sentFragment() {
        return this.sentFragment;
    }
    
    @Override
    public int getBufferSize() {
        return this.orb.getORBData().getGIOPFragmentSize();
    }
    
    @Override
    public void overflow(ByteBufferWithInfo byteBufferWithInfo) {
        MessageBase.setFlag(byteBufferWithInfo.byteBuffer, 2);
        this.queue.enqueue(byteBufferWithInfo);
        ByteBufferWithInfo byteBufferWithInfo2 = new ByteBufferWithInfo(this.orb, this);
        byteBufferWithInfo2.fragmented = true;
        ((CDROutputObject)this.outputObject).setByteBufferWithInfo(byteBufferWithInfo2);
        ((CDROutputObject)this.outputObject).getMessageHeader().createFragmentMessage().write((CDROutputObject)this.outputObject);
    }
    
    @Override
    public void sendMessage() {
        this.queue.enqueue(((CDROutputObject)this.outputObject).getByteBufferWithInfo());
        Iterator<Object> iterator = this.iterator();
        Connection connection = ((OutputObject)this.outputObject).getMessageMediator().getConnection();
        connection.writeLock();
        try {
            ByteBufferPool byteBufferPool = this.orb.getByteBufferPool();
            while (iterator.hasNext()) {
                ByteBufferWithInfo byteBufferWithInfo = (ByteBufferWithInfo)iterator.next();
                ((CDROutputObject)this.outputObject).setByteBufferWithInfo(byteBufferWithInfo);
                connection.sendWithoutLock((CDROutputObject)this.outputObject);
                this.sentFragment = true;
                if (this.debug) {
                    int identityHashCode = System.identityHashCode(byteBufferWithInfo.byteBuffer);
                    StringBuffer o = new StringBuffer(80);
                    o.append("sendMessage() - releasing ByteBuffer id (");
                    o.append(identityHashCode).append(") to ByteBufferPool.");
                    this.dprint(o.toString());
                }
                byteBufferPool.releaseByteBuffer(byteBufferWithInfo.byteBuffer);
                byteBufferWithInfo.byteBuffer = null;
            }
            this.sentFullMessage = true;
        }
        finally {
            connection.writeUnlock();
        }
    }
    
    @Override
    public void close() {
        Iterator<Object> iterator = this.iterator();
        ByteBufferPool byteBufferPool = this.orb.getByteBufferPool();
        while (iterator.hasNext()) {
            ByteBufferWithInfo byteBufferWithInfo = (ByteBufferWithInfo)iterator.next();
            if (byteBufferWithInfo != null && byteBufferWithInfo.byteBuffer != null) {
                if (this.debug) {
                    int identityHashCode = System.identityHashCode(byteBufferWithInfo.byteBuffer);
                    StringBuffer o = new StringBuffer(80);
                    o.append("close() - releasing ByteBuffer id (");
                    o.append(identityHashCode).append(") to ByteBufferPool.");
                    this.dprint(o.toString());
                }
                byteBufferPool.releaseByteBuffer(byteBufferWithInfo.byteBuffer);
                byteBufferWithInfo.byteBuffer = null;
            }
            else {
                continue;
            }
        }
    }
    
    private void dprint(String s) {
        ORBUtility.dprint("BufferManagerWriteCollect", s);
    }
    
    private Iterator iterator() {
        return new BufferManagerWriteCollect.BufferManagerWriteCollectIterator();
    }
    
    private class BufferManagerWriteCollectIterator implements Iterator
    {
        @Override
        public boolean hasNext() {
            return BufferManagerWriteCollect.this.queue.size() != 0;
        }
        
        @Override
        public Object next() {
            return BufferManagerWriteCollect.this.queue.dequeue();
        }
        
        @Override
        public void remove() {
            throw new UnsupportedOperationException();
        }
    }
}
